require_relative 'bamm_log/exception_middleware'
require_relative 'bamm_log/formatter'
require_relative 'bamm_log/version'

module BammLog

  module Logger

    def self.included(klass)
      klass.use BammLog::ExceptionMiddleware
      klass.include BammLog::Formatter

      @instance ||= if ENV['RACK_ENV'] != 'development'
        EnvLogger.new(klass)
      else
        DevLogger.new
      end
    end

    def self.instance
      @instance
    end

    # ------------------------------------------------------------- loggers

    class EnvLogger

      attr_accessor :logger

      def initialize(app)
        @logger = ::Logger.new(EnvLogger.log_file(app)).tap do |logger|
          ::Logger.class_eval { alias :write :'<<' }
          logger.level = ::Logger::INFO
        end
      end

      def self.log_file(app)
        @log_file ||= File.new("#{ENV['ROOT_PATH']}/log/#{ENV['RACK_ENV']}.log", 'a+').tap do |log_file|
          log_file.sync = true
        end
      end

      def method_missing(method, args, &block)
        logger.send(method, args, &block)
      end

    end

    class DevLogger

      def info(text)
        puts BammLog::Formatter.call_formatted(Time.now, text)
      end

    end

  end

end
