module BammLog

  class ExceptionMiddleware

    attr_reader :env, :request, :logger

    def initialize(app, options={})
      @app = app
    end

    def call(env)
      begin
        @request_id = SecureRandom.base64(24).tr('+/=lIO0', 'pqrsxyz')
        @start_time = Time.now
        @env = env

        build_request
        log_start
        log_params

        begin
          call = @app.call(env)
          @end_time = Time.now
          log_end
          call
        rescue Exception => e
          @end_time = Time.now
          log_exception(e)
          log_end
          @app.call(env)
        end
      rescue Exception => e
        log_exception(e)
      end
    end

    private

    def build_request
      @request = Rack::Request.new(env)
    end

    def request_params
      request.params.except(:route_info)
    end

    def log_start
      print "Started #{env['REQUEST_METHOD']} \"#{env['REQUEST_URI']}\""
    end

    def log_params
      print "Parameters: #{request_params.to_hash.inspect}"
    end

    def log_exception(e)
      print "Bamm! #{e.class}: #{e.message}"

      e.backtrace.each do |line|
        print "#{line}"
      end
    end

    def log_end
      print "Ended #{env['REQUEST_METHOD']} \"#{env['REQUEST_URI']}\" in #{request_time_ms} ms"
    end

    def request_time_ms
      (@end_time-@start_time).round(4)
    end

    def print(message)

      if ENV['RACK_ENV'] != 'development' && ENV['RACK_ENV'] != 'test'
        message = "[#{@request_id}] #{message}"
      end

      BammLog::Logger.instance.info message
    end

    def request_body_params?
      ['POST', 'PUT', 'PATCH'].include? env['REQUEST_METHOD']
    end

  end

end
